package geniusweb.references;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.Arrays;
import java.util.List;

import org.junit.Test;

import tudelft.utilities.junit.GeneralTests;

public class ParametersTest extends GeneralTests<Parameters> {

	private final Parameters params1 = new Parameters();
	private final Parameters params1a = new Parameters();
	private final Parameters params2 = new Parameters().with("a", 1).with("b",
			2d);

	@Override
	public List<List<Parameters>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(params1, params1a),
				Arrays.asList(params2));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList("\\{\\}", "\\{a=1, b=2.0\\}");
	}

	@Test(expected = IllegalArgumentException.class)
	public void getWrongClass() {
		params2.get("a", String.class);
	}

	@Test
	public void getInteger() {
		assertEquals((Integer) 1, params2.get("a", Integer.class));
		assertEquals((Double) 2d, params2.get("b", Double.class));
	}

	@Test
	public void getDouble() {
		assertEquals((Double) 2d, params2.getDouble("b", 7d, null, null));
		assertEquals((Double) 2d, params2.getDouble("b", 7d, 0d, 4d));

		// test outside range
		assertEquals((Double) 7d, params2.getDouble("b", 7d, 4d, 6d));
		assertEquals((Double) 7d, params2.getDouble("b", 7d, -2d, 1d));

		// test fallback null
		assertEquals(null, params2.getDouble("b", null, 4d, 6d));

		// a is integer!
		assertEquals((Double) 1d, params2.getDouble("a", 0d, -0d, 2d));

	}

	@Test
	public void getDoubleWithErrors() {
		// a does not contain a double
		assertEquals((Double) 7d, params2.getDouble("a", 7d, 4d, 6d));
		// c does not exist
		assertEquals((Double) 7d, params2.getDouble("c", 7d, 4d, 6d));

		// a=1, which is outside [min, max]. Therefore default value should be
		// returned.
		assertEquals(null, params2.getDouble("a", null, 2d, 3d));

	}

	@Test
	public void withTest() {
		Parameters p = params2.with("a", 2);
		assertEquals(1, params2.get("a"));
		assertEquals(2d, params2.get("b"));
		assertEquals(2, p.get("a"));
		assertEquals(2d, p.get("b"));
	}

	@Test
	public void withParamsTest() {
		Parameters upd = new Parameters().with("a", 2);
		Parameters p = params2.with(upd);
		assertEquals(1, params2.get("a"));
		assertEquals(2d, params2.get("b"));
		assertEquals(2, p.get("a"));
		assertEquals(2d, p.get("b"));
	}

	@Test
	public void isEmptyTest() {
		assertTrue(params1.isEmpty());
		assertFalse(params2.isEmpty());
	}

	@Test
	public void containsKeyTest() {
		assertTrue(params2.containsKey("a"));
		assertFalse(params2.containsKey("c"));
	}
}
